@@ -5,46 +5,16 @@ import pycparser.c_ast as c_ast
 try:
     import pycparser.ply.yacc as yacc
 except ImportError:
-    import ply.yacc as yacc
+    import ply.yacc as yacc  # noqa: F401
+from pycparser.plyparser import parameterized, template
 
 
 class CParserBase(pycparser.c_parser.CParser):
-    OPT_RULES = [
-        'abstract_declarator',
-        'assignment_expression',
-        'declaration_list',
-        'declaration_specifiers',
-        'designation',
-        'expression',
-        'identifier_list',
-        'init_declarator_list',
-        'initializer_list',
-        'parameter_type_list',
-        'specifier_qualifier_list',
-        'block_item_list',
-        'type_qualifier_list',
-        'struct_declarator_list'
-    ]
-
-    def __init__(self, yacc_debug=False):
-        self.clex = self.lexer_class(
-                error_func=self._lex_error_func,
-                on_lbrace_func=self._lex_on_lbrace_func,
-                on_rbrace_func=self._lex_on_rbrace_func,
-                type_lookup_func=self._lex_type_lookup_func)
-
-        self.clex.build()
-        self.tokens = self.clex.tokens
-
-        for rule in self.OPT_RULES:
-            self._create_opt_rule(rule)
-
-        self.ext_start_symbol = "translation_unit_or_empty"
-
-        self.cparser = yacc.yacc(
-            module=self,
-            start=self.ext_start_symbol,
-            debug=yacc_debug, write_tables=False)
+    def __init__(self, **kwds):
+        kwds['lexer'] = self.lexer_class
+        kwds['lextab'] = 'pycparserext.lextab'
+        kwds['yacctab'] = 'pycparserext.yacctab'
+        pycparser.c_parser.CParser.__init__(self, **kwds)
 
     def parse(self, text, filename='', debuglevel=0,
             initial_type_symbols=set()):
@@ -77,6 +47,10 @@ class TypeList(c_ast.Node):
             nodelist.append(("types[%d]" % i, child))
         return tuple(nodelist)
 
+    def __iter__(self):
+        for child in (self.types or []):
+            yield child
+
     attr_names = ()
 
 
@@ -87,6 +61,12 @@ class AttributeSpecifier(c_ast.Node):
     def children(self):
         return [("exprlist", self.exprlist)]
 
+    def __iter__(self):
+        # Do not return anything, but yield is necessary to keep this function
+        # a generator
+        return
+        yield
+
     attr_names = ()
 
 
@@ -112,6 +92,16 @@ class Asm(c_ast.Node):
             nodelist.append(("clobbered_regs", self.clobbered_regs))
         return tuple(nodelist)
 
+    def __iter__(self):
+        if self.template is not None:
+            yield self.template
+        if self.output_operands is not None:
+            yield self.output_operands
+        if self.input_operands is not None:
+            yield self.input_operands
+        if self.clobbered_regs is not None:
+            yield self.clobbered_regs
+
     attr_names = ('asm_keyword',)
 
 
@@ -123,6 +113,12 @@ class PreprocessorLine(c_ast.Node):
     def children(self):
         return ()
 
+    def __iter__(self):
+        # Do not return anything, but yield is necessary to keep this function
+        # a generator
+        return
+        yield
+
     attr_names = ("contents",)
 
 
@@ -137,6 +133,10 @@ class TypeOfDeclaration(c_ast.Node):
             nodelist.append(("declaration", self.declaration))
         return tuple(nodelist)
 
+    def __iter__(self):
+        if self.declaration is not None:
+            yield self.declaration
+
     attr_names = ()
 
 
@@ -151,6 +151,10 @@ class TypeOfExpression(c_ast.Node):
             nodelist.append(("expr", self.expr))
         return tuple(nodelist)
 
+    def __iter__(self):
+        if self.expr is not None:
+            yield self.expr
+
     attr_names = ()
 
 
@@ -199,6 +203,16 @@ class FuncDeclExt(c_ast.Node):
             nodelist.append(("asm", self.asm))
         return tuple(nodelist)
 
+    def __iter__(self):
+        if self.args is not None:
+            yield self.args
+        if self.type is not None:
+            yield self.type
+        if self.attributes is not None:
+            yield self.attributes
+        if self.asm is not None:
+            yield self.asm
+
     attr_names = ()
 
 # }}}
@@ -206,6 +220,7 @@ class FuncDeclExt(c_ast.Node):
 
 # {{{ attributes
 
+@template
 class _AttributesMixin(object):
     def p_attributes_opt_1(self, p):
         """ attributes_opt : attribute_decl attributes_opt
@@ -247,8 +262,9 @@ class _AttributesMixin(object):
 
     # {{{ /!\ names must match C parser to override
 
-    def p_declarator_1(self, p):
-        """ declarator  : direct_declarator attributes_opt
+    @parameterized(('id', 'ID'), ('typeid', 'TYPEID'), ('typeid_noparen', 'TYPEID'))
+    def p_xxx_declarator_1(self, p):
+        """ xxx_declarator  : direct_xxx_declarator attributes_opt
         """
         if p[2].exprs:
             if isinstance(p[1], c_ast.ArrayDecl):
@@ -268,19 +284,18 @@ class _AttributesMixin(object):
         p[0] = p[1]
 
     # }}}
-    def p_declarator_2(self, p):
-        """declarator  : pointer direct_declarator attributes_opt
-                       | pointer attributes_opt direct_declarator
+
+    @parameterized(('id', 'ID'), ('typeid', 'TYPEID'), ('typeid_noparen', 'TYPEID'))
+    def p_xxx_declarator_2(self, p):
+        """ xxx_declarator  : pointer direct_xxx_declarator attributes_opt
+                            | pointer attributes_opt direct_xxx_declarator
         """
         if hasattr(p[3], "exprs"):
             attr_decl = p[3]
             decl = p[2]
-        elif hasattr(p[2], "exprs"):
+        else:
             attr_decl = p[2]
             decl = p[3]
-        else:
-            attr_decl = None
-            decl = p[2]
 
         if not attr_decl.exprs:
             attr_decl = None
@@ -326,27 +341,27 @@ class _AsmMixin(object):
     def p_asm_1(self, p):
         """ asm_no_semi : asm_keyword LPAREN asm_argument_expression_list RPAREN
         """
-        p[0] = Asm(p[1], p[3], None, None, None, coord=self._coord(p.lineno(1)))
+        p[0] = Asm(p[1], p[3], None, None, None, coord=self._coord(p.lineno(2)))
 
     def p_asm_2(self, p):
         """ asm_no_semi : asm_keyword LPAREN asm_argument_expression_list COLON \
                 asm_argument_expression_list RPAREN
         """
-        p[0] = Asm(p[1], p[3], p[5], None, None, coord=self._coord(p.lineno(1)))
+        p[0] = Asm(p[1], p[3], p[5], None, None, coord=self._coord(p.lineno(2)))
 
     def p_asm_3(self, p):
         """ asm_no_semi : asm_keyword LPAREN asm_argument_expression_list COLON \
                 asm_argument_expression_list COLON asm_argument_expression_list \
                 RPAREN
         """
-        p[0] = Asm(p[1], p[3], p[5], p[7], None, coord=self._coord(p.lineno(1)))
+        p[0] = Asm(p[1], p[3], p[5], p[7], None, coord=self._coord(p.lineno(2)))
 
     def p_asm_4(self, p):
         """ asm_no_semi : asm_keyword LPAREN asm_argument_expression_list COLON \
                 asm_argument_expression_list COLON asm_argument_expression_list \
                 COLON asm_argument_expression_list RPAREN
         """
-        p[0] = Asm(p[1], p[3], p[5], p[7], p[9], coord=self._coord(p.lineno(1)))
+        p[0] = Asm(p[1], p[3], p[5], p[7], p[9], coord=self._coord(p.lineno(2)))
 
     def p_asm_keyword(self, p):
         """ asm_keyword : __ASM__ asm_volatile
@@ -381,14 +396,17 @@ class _AsmMixin(object):
 #        p[0] = p[1]
 
 
+@template
 class _AsmAndAttributesMixin(_AsmMixin, _AttributesMixin):
     # {{{ /!\ names must match C parser to override
 
-    def p_direct_declarator_5(self, p):
-        """ direct_declarator   : direct_declarator LPAREN parameter_type_list \
-                                        RPAREN asm_opt attributes_opt
-                                | direct_declarator LPAREN identifier_list_opt \
-                                        RPAREN asm_opt attributes_opt
+    @parameterized(('id', 'ID'), ('typeid', 'TYPEID'), ('typeid_noparen', 'TYPEID'))
+    def p_direct_xxx_declarator_6(self, p):
+        """ direct_xxx_declarator   : direct_xxx_declarator LPAREN parameter_type_list \
+                                            RPAREN asm_opt attributes_opt
+                                    | direct_xxx_declarator \
+                                            LPAREN identifier_list_opt RPAREN \
+                                            asm_opt attributes_opt
         """
         func = FuncDeclExt(
             args=p[3],
@@ -480,6 +498,7 @@ class GnuCParser(_AsmAndAttributesMixin, CParserBase):
                         | selection_statement
                         | iteration_statement
                         | jump_statement
+                        | pppragma_directive
                         | gnu_statement_expression
         """
         p[0] = p[1]
